#
#    TSP formulation by Van Vyve and Wolsey
#
#    WS 2011, TU Berlin, Computational Integer Programming
#    Template file for exercise 01 on exercise sheet 03
#    2011/11/10, Ralf Borndoerfer

# definition of the data file to be read
#param FILE := "germany18.dat";
#param FILE := "att48.dat";
#param FILE := "berlin52.dat";
#param FILE := "brazil58.dat";
#param FILE := "eil101.dat";

set V   := { read FILE as "<1s>" comment "#" };
set E   := { <i,j> in V * V with i != j };

param px[V] := read FILE as "<1s> 2n" comment "#";
param py[V] := read FILE as "<1s> 3n" comment "#";

defnumb dist_euc(a,b) := sqrt((px[a] - px[b])^2 + (py[a] - py[b])^2);
defnumb dist_max(a,b) := max(abs(px[a] - px[b]),abs(py[a] - py[b]));
defnumb dist_man(a,b) := (abs(px[a] - px[b]) + abs(py[a] - py[b]));

defnumb dist(a,b) := dist_euc(a,b);

# local neighborhoods for all nodes
param LOC := 13;
set W  := { <i> in V with i != ord(V,1,1) };
set NV[<i> in W] := { i } + argmin(LOC) <j> in W \ { i, ord(V,1,1) } : dist(i,j) + dist(j,i);
set NE[<i> in W] := { <j,k> in E with <j> in NV[i] or <k> in NV[i] };
set VxNE := union <i> in W : ({ i } * NE[i]);

# variables
var x[E] binary;
var u[<j> in V] integer 
   >= if j == ord(V,1,1) then 1 else 2 end  
   <= if j == ord(V,1,1) then 1 else card(V) end;
var w[VxNE] binary;

minimize cost: sum <i,j> in E : dist(i,j) * x[i,j];

# degree constraints
subto two_in:
   forall <j> in V do sum <i,j> in E : x[i,j] == 1;
subto two_out:
   forall <i> in V do sum <i,j> in E : x[i,j] == 1;

# MTZ constraints
subto imply_order:
   forall <i,j> in E with j != ord(V,1,1) do
      u[i] - u[j] + 1  <= card(V) * ( 1 - x[i,j]);

# local flow constraints

# coupling constraints


